﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking;

namespace HeuristicLab.Encodings.PackingEncoding.PackingSequence {
  [Item("SequenceMove", "An abstract move for a packingSequence encoded solution.")]
  [StorableClass]
  public abstract class SequenceMove : Item, IPackingMove {
    [Storable]
    public PackingSequenceEncoding PackingSequence { get; protected set; }

    [StorableConstructor]
    protected SequenceMove(bool deserializing) : base(deserializing) { }
    protected SequenceMove(SequenceMove original, Cloner cloner)
      : base(original, cloner) {
        if (original.PackingSequence != null)
          this.PackingSequence = cloner.Clone(original.PackingSequence);
    }
    public SequenceMove(PackingSequenceEncoding packingSequence)
      : base() {
        PackingSequence = packingSequence;
    }

    public abstract IPackingSolutionEncoding GetSolutionAfterMove();
    public abstract Type GetMoveAttributeType();
    public abstract SequenceMoveAttribute GetAttribute(double quality);
    public abstract bool BelongsToAttribute (SequenceMoveAttribute attribute, bool hardCriterion);
  }
}